package gs.test.v0.definition

import cats.Show
import cats.effect.Async

/** Each group is comprised of a list of [[Test]]. This list may be empty.
  *
  * Groups are essentially metadata for tests for viewing/organization purposes.
  *
  * @param name
  *   The group name. Not considered to be unique.
  * @param documentation
  *   Arbitrary documentation for this group of tests.
  * @param testTags
  *   Set of tags applied to all [[TestDefinition]] within the group.
  * @param testMarkers
  *   Set of markers applied to all [[TestDefinition]] within the group.
  * @param tests
  *   The list of tests in this group.
  */
final class TestGroupDefinition[F[_]: Async](
  val name: TestGroupDefinition.Name,
  val documentation: Option[String],
  val testTags: List[Tag],
  val testMarkers: List[Marker],
  val beforeGroup: Option[F[Unit]],
  val afterGroup: Option[F[Unit]],
  val beforeEachTest: Option[F[Unit]],
  val afterEachTest: Option[F[Unit]],
  val tests: List[TestDefinition[F]]
)

object TestGroupDefinition:

  /** Opaque type representing names that may be assigned to test groups.
    */
  opaque type Name = String

  object Name:

    /** Instantiate a new [[TestGroup.Name]]. This name is not unique, has no
      * constraints, and only exists for display purposes.
      *
      * @param name
      *   The candidate string.
      * @return
      *   The new [[TestGroup.Name]] instance.
      */
    def apply(name: String): Name = name

    given CanEqual[Name, Name] = CanEqual.derived

    given Show[Name] = name => name

  end Name

end TestGroupDefinition
