package gs.test.v0.definition.pos

import cats.data.Kleisli
import cats.effect.IO
import cats.effect.kernel.Resource
import gs.test.v0.IOSuite
import gs.test.v0.definition.*
import munit.*
import natchez.EntryPoint
import natchez.Kernel
import natchez.Span
import natchez.Span.Options
import natchez.Trace
import natchez.Trace.Implicits.noop

/** These tests are sensitive to changes, even in formatting! They are looking
  * for specific line numbers in this source code, so any sort of newline that
  * occurs prior to the sub-test-definitions will cause the checked positions to
  * change!
  */
class SourcePositionTests extends IOSuite:

  import SourcePositionTests.*

  val ep: EntryPoint[IO] = new EntryPoint[IO] {
    override def root(
      name: String,
      options: Options
    ): Resource[IO, Span[IO]] =
      Resource.make(IO(Span.noop[IO]))(_ => IO.unit)

    override def continue(
      name: String,
      kernel: Kernel,
      options: Options
    ): Resource[IO, Span[IO]] =
      Resource.make(IO(Span.noop[IO]))(_ => IO.unit)

    override def continueOrElseRoot(
      name: String,
      kernel: Kernel,
      options: Options
    ): Resource[IO, Span[IO]] =
      Resource.make(IO(Span.noop[IO]))(_ => IO.unit)
  }

  test("should provide the source position of a failed check") {
    lookForSourcePosition(new G1, 73)
  }

  test("should provide the source position of an explicit failure") {
    lookForSourcePosition(new G2, 82)
  }

  private def lookForSourcePosition(
    groupDef: TestGroup[TracedIO],
    line: Int
  ): Unit =
    val group = groupDef.compile()
    group.tests match
      case t1 :: Nil =>
        ep.root("unit-test")
          .use { span =>
            t1.unitOfWork.run(span).map {
              case Left(TestFailure.AssertionFailed(_, _, _, pos)) =>
                assertEquals(pos.file.endsWith(SourceFileName), true)
                assertEquals(pos.line, line)
              case Left(TestFailure.TestRequestedFailure(_, pos)) =>
                assertEquals(pos.file.endsWith(SourceFileName), true)
                assertEquals(pos.line, line)
              case _ => fail("Sub-test did not fail as expected.")
            }
          }
          .unsafeRunSync()
      case _ =>
        fail("Wrong number of tests - position tests need one test per group.")

object SourcePositionTests:

  type TracedIO[A] = Kleisli[IO, Span[IO], A]

  class G1 extends TestGroup[TracedIO]:
    override def name: String = "G1"

    test(pid"t1", "pos").pure {
      check(false).isTrue()
    }

  end G1

  class G2 extends TestGroup[TracedIO]:
    override def name: String = "G2"

    test(pid"t2", "pos").pure {
      gs.test.v0.definition.fail("Expected Failure")
    }

  end G2

  val SourceFileName: String =
    "modules/api-definition/src/test/scala/gs/test/v0/definition/pos/SourcePositionTests.scala"

end SourcePositionTests
