package gs.test.v0.execution

import cats.Show
import gs.test.v0.definition.Marker
import gs.test.v0.definition.PermanentId
import gs.test.v0.definition.Tag
import gs.test.v0.definition.TestFailure
import gs.test.v0.definition.pos.SourcePosition
import gs.uuid.v0.UUID
import scala.concurrent.duration.FiniteDuration

/** Represents a single _Test Execution_. Each _Test Execution_ represents
  * evaluating the unit of work for some
  * [[gs.test.v0.definition.TestDefinition]] exactly once. It describes the
  * result of the test.
  *
  * @param id
  *   The unique identifier for this execution.
  * @param permanentId
  *   The [[gs.test.v0.definition.PermanentId]] for the
  *   [[gs.test.v0.definition.TestDefinition]] that was executed.
  * @param documentation
  *   Documentation for the test that was executed.
  * @param tags
  *   Tags for the test that was executed.
  * @param markers
  *   Markers for the test that was executed.
  * @param result
  *   The result of the test.
  * @param traceId
  *   The 128-bit trace identifier used for this test.
  * @param sourcePosition
  *   The position, in source code, of the test that was executed.
  * @param duration
  *   The amount of time it took to execute the test. This amount includes setup
  *   and cleanup.
  */
case class TestExecution(
  id: TestExecution.Id,
  permanentId: PermanentId,
  documentation: Option[String],
  tags: List[Tag],
  markers: List[Marker],
  result: Either[TestFailure, Any],
  traceId: UUID,
  sourcePosition: SourcePosition,
  duration: FiniteDuration
)

object TestExecution:

  /** Opaque type for UUID representing a unique test execution.
    */
  opaque type Id = UUID

  object Id:

    /** Instantiate a new [[TestExecution.Id]].
      *
      * @param value
      *   The underlying UUID.
      * @return
      *   The new [[TestExecution.Id]] instance.
      */
    def apply(value: UUID): Id = value

    given CanEqual[Id, Id] = CanEqual.derived

    given Show[Id] = id => id.toUUID().withoutDashes()

    extension (id: Id)
      /** @return
        *   The underlying UUID.
        */
      def toUUID(): UUID = id

  end Id

end TestExecution
